/*
 * eth_ecat_pdo_fmmu.cpp
 *
 *  Created on: May 16, 2023
 *      Author: algin
 */

#include "ethernet_industry/CoE/eth_ecat_pdo_fmmu.hpp"

#include <algorithm>

namespace free_rtos {

namespace ecat_pdo_fmmu {

void EthEcatPdoFMMU::init() {
    std::vector<ecat_buffer::EcatBufferSlave>& buffer_slaves = ecat_buffer_.get_buffer_slaves();

    pdo_fmmu_slaves_.reserve(buffer_slaves.size());

    for(ecat_buffer::EcatBufferSlave& buffer_slave : buffer_slaves) {
        pdo_fmmu_slaves_.emplace_back(EcatPdoFMMUSlave{buffer_slave});
    }
}

void EthEcatPdoFMMU::wait_op() {
    free_rtos::Semaphore& init_sem = ecat_buffer_.get_ecat().get_init_sem();
    free_rtos::Semaphore& process_sem = ecat_buffer_.get_ecat().get_process_sem();

    process_sem.pend();

    uint32_t logical_full_length_write = ecat_buffer_.get_fmmu_global_properties().logical_full_length_write;
    uint32_t logical_full_length_read = ecat_buffer_.get_fmmu_global_properties().logical_full_length_read;
    std::vector<uint8_t> process_data(std::min(logical_full_length_write, logical_full_length_read));

    read(0, process_data);
/*
    for(uint8_t& byte : process_data) {
        DebugP_log("0x%01x", byte);
    }
    DebugP_log("\r\n");
*/
    write(0, process_data);

    init_sem.post();
    process_sem.pend();

    for(uint32_t i = 0; i < 250; i++) {
        read(0, process_data);
/*
        for(uint8_t& byte : process_data) {
            DebugP_log("0x%01x", byte);
        }
        DebugP_log("\r\n");
*/
        write(0, process_data);
    }

    init_sem.post();
}

void EthEcatPdoFMMU::process_write_queue(uint8_t* process_data, uint32_t len) {
    custom_promise::IPromise *next;

    mutex_write_.lock();

    next = queue_write_.get_next();
    queue_write_.detach();

    mutex_write_.unlock();

    while(next != nullptr) {
        next->set_value(process_data, len);
        next = next->get_next();
    }
}

void EthEcatPdoFMMU::process_read_queue(uint8_t* process_data, uint32_t len) {
    custom_promise::IPromise *next;

    mutex_read_.lock();

    next = queue_read_.get_next();
    queue_read_.detach();

    mutex_read_.unlock();

    while(next != nullptr) {
        next->set_value(process_data, len);
        next = next->get_next();
    }
}

void EthEcatPdoFMMU::process() {
    wait_op();

    uint32_t logical_full_length_write = ecat_buffer_.get_fmmu_global_properties().logical_full_length_write;
    uint32_t logical_full_length_read = ecat_buffer_.get_fmmu_global_properties().logical_full_length_read;
    std::vector<uint8_t> process_data(std::min(logical_full_length_write, logical_full_length_read));
    free_rtos::Timer& ecat_timer = ecat_buffer_.get_ecat().get_ecat_timer();

    while(1) {
        ecat_timer.Wait();

        read(0, process_data);
/*
        for(uint8_t& byte : process_data) {
            DebugP_log("0x%01x", byte);
        }
        DebugP_log("\r\n");
*/
        process_read_queue(process_data.data(), process_data.size());
        process_write_queue(process_data.data(), process_data.size());

        write(0, process_data);
    }
}

}

}
