/*
 * eth_ecat_api.hpp
 *
 *  Created on: May 29, 2023
 *      Author: algin
 */

#ifndef FREE_RTOS_ETHERNET_INDUSTRY_ETH_ECAT_API_HPP_
#define FREE_RTOS_ETHERNET_INDUSTRY_ETH_ECAT_API_HPP_

#define COMX 1

#include "ethernet/eth.hpp"
#include "ethernet_industry/eth_ecat.hpp"
#include "ethernet_industry/eth_ecat_buffer.hpp"
#include "ethernet_industry/CoE/eth_ecat_sdo_mailbox.hpp"
#include "ethernet_industry/CoE/eth_ecat_pdo_fmmu.hpp"

namespace free_rtos {

class EthEcatApi {
public:
    static bool init(Eth& eth, TEthMacPorts port_id, uint16_t address_base);
    static void process(); // Внутри бесконечный цикл. Запускать в отдельном потоке

    static std::vector<ecat_buffer::PDOMap>& get_ecat_pdo_map();

    template<typename... DataTypes>
    static ecat_sdo_mailbox::CompleteSize sdo_write(size_t slave_index, uint16_t index, uint8_t subindex, DataTypes&... data) {
        return get_ecat_sdo_mailbox().sdo_write<command::FP, DataTypes...>(slave_index, index, subindex, data...);
    }

    template<typename... DataTypes>
    static ecat_sdo_mailbox::CompleteSize sdo_read(size_t slave_index, uint16_t index, uint8_t subindex, DataTypes&... data) {
        return get_ecat_sdo_mailbox().sdo_read<command::FP, DataTypes...>(slave_index, index, subindex, data...);
    }

    template<typename... DataTypes>
    static void pdo_write(address::Offset offset, DataTypes&... data) {
        get_ecat_pdo_fmmu().pdo_write<DataTypes...>(offset, data...);
    }

    template<typename... DataTypes>
    static void pdo_read(address::Offset offset, DataTypes&... data) {
        get_ecat_pdo_fmmu().pdo_read<DataTypes...>(offset, data...);
    }

    static void pdo_write_async(custom_promise::IPromise& promise) {
        get_ecat_pdo_fmmu().pdo_write_async(promise);
    }

    static void pdo_read_async(custom_promise::IPromise& promise) {
        get_ecat_pdo_fmmu().pdo_read_async(promise);
    }

private:
    static Eth *eth_;

    static EthEcat& get_ecat();
    static ecat_buffer::EthEcatBuffer& get_ecat_buffer_sdo();
    static ecat_buffer::EthEcatBuffer& get_ecat_buffer_pdo();
    static ecat_sdo_mailbox::EthEcatSdoMailbox& get_ecat_sdo_mailbox();
    static ecat_pdo_fmmu::EthEcatPdoFMMU& get_ecat_pdo_fmmu();
};

}

#endif /* FREE_RTOS_ETHERNET_INDUSTRY_ETH_ECAT_API_HPP_ */
