/*
 * eth_ecat_custom_promise.hpp
 *
 *  Created on: Jun 1, 2023
 *      Author: algin
 */

#ifndef FREE_RTOS_ETHERNET_INDUSTRY_COE_ETH_ECAT_CUSTOM_PROMISE_HPP_
#define FREE_RTOS_ETHERNET_INDUSTRY_COE_ETH_ECAT_CUSTOM_PROMISE_HPP_

#include <cstdint>

#include "semaphore/semaphore.hpp"

#include "ethernet_industry/eth_ecat_queue.hpp"
#include "ethernet_industry/eth_ecat_datagram.hpp"
#include "ethernet_industry/eth_ecat_command.hpp"
#include "ethernet_industry/eth_ecat_custom_tuple.hpp"
#include "ethernet_industry/eth_ecat_packer.hpp"

namespace free_rtos {

namespace custom_promise {

/*
// Функтор для обхода и упаковки датаграмм в custom_tuple
struct DatagramPackFunctor : public PackFunctor {
    DatagramPackFunctor(uint8_t *raw)
    : PackFunctor{raw} { }

    using PackFunctor::operator ();

    template<typename CommandT, typename... DataTypes>
    void operator()(datagram::EcatDatagram<CommandT, DataTypes...>& data) {
        raw = data.pack(raw);
    }

    template<typename CommandT, typename... DataTypes>
    void operator()(std::vector< datagram::EcatDatagram<CommandT, DataTypes...> >& data) {
        for(uint16_t i = 1; i < data.size(); i++) {
            data[i - 1] + data[i];

            raw = data[i - 1].pack(raw);
        }

        raw = data[data.size() - 1].pack(raw);
    }
};
*/

// Функтор для обхода и распаковки датаграмм в custom_tuple
struct DatagramUnpackFunctor : public UnpackFunctor {
    DatagramUnpackFunctor(uint8_t *raw)
    : UnpackFunctor{raw} { }

    using UnpackFunctor::operator ();

    template<typename CommandT, typename... DataTypes>
    void operator()(datagram::EcatDatagram<CommandT, DataTypes...>& data) {
        raw = data.unpack(raw);
    }

    template<typename CommandT, typename... DataTypes>
    void operator()(std::vector< datagram::EcatDatagram<CommandT, DataTypes...> >& data) {
        for(uint16_t i = 1; i < data.size(); i++) {
            data[i - 1] + data[i];

            raw = data[i - 1].unpack(raw);
        }

        raw = data[data.size() - 1].unpack(raw);
    }
};

template<typename... DataTypes>
class Future {
public:
    Future(DataTypes&... data)
    : data_tuple_{data...} { }

    bool is_ready() {
        return ready_;
    }

    custom_tuple<DataTypes&...> get() {
        ready_ = false;
        sem_.pend();

        return data_tuple_;
    }

    void pack(uint8_t* raw) {
        PackFunctor functor{raw};

        for_each(data_tuple_, functor);

        sem_.post();
        ready_ = true;
    }

    void unpack(uint8_t* raw) {
        UnpackFunctor functor{raw};

        for_each(data_tuple_, functor);

        sem_.post();
        ready_ = true;
    }

private:
    custom_tuple<DataTypes&...> data_tuple_;
    free_rtos::Semaphore sem_;
    bool ready_{false};
};

class IPromise {
public:
    IPromise(address::Offset offset = 0)
    : offset_{offset} { }

    IPromise* get_next() {
        queue::QueueEntity<IPromise>* next = queue_entity_.get_next();

        if(next == nullptr) {
            return nullptr;
        }

        return next->get_data();
    }

    queue::QueueEntity<IPromise>& get_queue_entity() {
        return queue_entity_;
    }

    IPromise& operator+(IPromise &next) {
        queue_entity_ + next.get_queue_entity();

        return next;
    }

    void detach() {
        queue_entity_.detach();
    }

    virtual void set_value(uint8_t* process_data, uint32_t len) = 0;

protected:
    address::Offset offset_{0};

private:
    queue::QueueEntity<IPromise> queue_entity_{this};
};

template<typename... DataTypes>
class WritePromise : public IPromise {
public:
    WritePromise(address::Offset offset, DataTypes&... data)
    : IPromise{offset}
    , future_{data...} { }

    Future<DataTypes...>& get_future() {
        return future_;
    }

    virtual void set_value(uint8_t* process_data, uint32_t len) override {
        future_.pack(process_data + offset_);
    }

private:
    Future<DataTypes...> future_;
};

template<typename... DataTypes>
class ReadPromise : public IPromise {
public:
    ReadPromise(address::Offset offset, DataTypes&... data)
    : IPromise{offset}
    , future_{data...} { }

    Future<DataTypes...>& get_future() {
        return future_;
    }

    virtual void set_value(uint8_t* process_data, uint32_t len) override {
        future_.unpack(process_data + offset_);
    }

private:
    Future<DataTypes...> future_;
};

} // namespace custom_promise

} // namespace free_rtos

#endif /* FREE_RTOS_ETHERNET_INDUSTRY_COE_ETH_ECAT_CUSTOM_PROMISE_HPP_ */
